#ifndef OGLUTIL_H
#define OGLUTIL_H

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

// Include the standard OpenGL headers
#include "gl\gl.h"     
#include "gl\glu.h"
#include "gl\glaux.h"

#include "math.h"    // For sqrt
#include <stdexcept> // For runtime_error exception

//******************************************************************************
// OpenGL reusable utility template classes GLpoint4 and GLvector3  
// By G.Bavestrelli, Copyright Techint S.p.A. 1998-1999
// Any feedback is welcome, you can contact me at giovanni-bavestrelli@pomini.it
// This is the version without tricks and public variables but...
// THIS VERSION IS NOT WELL TESTED !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//******************************************************************************

// This is a vector in three dimensional space
template <class Numeric>
class GLvector3
{

  enum { X,Y,Z };

  Numeric V[3];

public:

  GLvector3()
  {
     memset(V,0,sizeof(V));
  }

  GLvector3(Numeric x, Numeric y, Numeric z)
  {
    V[X]=x; V[Y]=y; V[Z]=z;
  }

  // The default copy constructor works fine and 
  // is faster than a member by member assignment
  // The same is true for the assigment operator


  explicit GLvector3(const Numeric * v)
  {
     ASSERT(v);
     memcpy(V,v,sizeof(V));
  }

  const Numeric * ptr() const
  {
     return V;
  }

  GLvector3 & operator =(const Numeric * v)
  {
      ASSERT(v);
      memcpy(V,v,sizeof(V));
      return *this; 
  }

  GLvector3 operator - ()
  {
      return GLvector3<Numeric>(-V[X],-V[Y],-V[Z]); 
  }

  double GetLength() const 
  { 
      return sqrt(static_cast<double>(V[X]*V[X]+V[Y]*V[Y]+V[Z]*V[Z])); 
  }

  bool Normalize() 
  {
     double Length=GetLength();
     if (Length==0.0) return false;
     V[X]/=Length; V[Y]/=Length; V[Z]/=Length; 
     return true;
  }

  GLvector3 GetNormal() const 
  {
     double Length=GetLength();
     if (Length==0.0) return GLvector3();
     else return GLvector3(V[X]/Length,V[Y]/Length,V[Z]/Length);
  }

  // Accessors
  Numeric x() const { return V[X]; }
  Numeric y() const { return V[Y]; }
  Numeric z() const { return V[Z]; } 

  // Modifiers
  void x(Numeric x) { V[X]=x; }
  void y(Numeric y) { V[Y]=y; }
  void z(Numeric z) { V[Z]=z; }

};

template <class Numeric>
inline bool operator == (const GLvector3<Numeric> & V1, const GLvector3<Numeric> & V2)
{
    return V1.x()==V2.x() && V1.y()==V2.y() && V1.z()==V2.z();
}

template <class Numeric>
inline bool operator != (const GLvector3<Numeric> & V1, const GLvector3<Numeric> & V2)
{
    return !(V1==V2);
}


// This is a homogeneous coordinate in three dimensional space
template <class Numeric>
class GLpoint4
{
   enum { X,Y,Z,W };

   Numeric V[4];

 public:

   GLpoint4()
   {
      static Numeric v[4]={0,0,0,1};
      memcpy(V,v,sizeof(V)); 
   }

   // The default copy constructor works fine and 
   // is faster than a member by member assignment
   // The same is true for the assigment operator

   GLpoint4(Numeric x,Numeric y,Numeric z=0,Numeric w=1)
   { 
      V[X]=x; V[Y]=y; V[Z]=z; V[W]=w; 
   }

   explicit GLpoint4(const Numeric * v)
   { 
      ASSERT(v);
      memcpy(V,v,sizeof(V)); 
   }
 
   const Numeric * ptr() const 
   { 
      return V; 
   }

   GLpoint4 & operator = (const Numeric * v)
   { 
      ASSERT(v);
      memcpy(V,v,sizeof(V)); 
      return *this; 
   }

   GLpoint4 operator - ()
   {
      return GLpoint4<Numeric>(-V[X],-V[Y],-V[Z],V[W]); 
   }

   bool Normalize()
   {
     if (V[W]==0) return false;
     if (V[W]==1) return true;
     V[X]/=V[W]; V[Y]/=V[W]; V[Z]/=V[Z]; V[W]=1; 
     return true;
   }

   // Accessors
   Numeric x() const { return V[X]; }
   Numeric y() const { return V[Y]; }
   Numeric z() const { return V[Z]; } 
   Numeric w() const { return V[W]; } 

   // Modifiers
   void x(Numeric x) { V[X]=x; }
   void y(Numeric y) { V[Y]=y; }
   void z(Numeric z) { V[Z]=z; }
   void w(Numeric w) { V[W]=w; }

};

template <class Numeric>
inline bool operator == (const GLpoint4<Numeric> & C1, const GLpoint4<Numeric> & C2)
{
    return C1.x()==C2.x() && C1.y()==C2.y() && C1.z()==C2.z() && C1.w()==C2.w();
}

template <class Numeric>
inline bool operator != (const GLpoint4<Numeric> & C1, const GLpoint4<Numeric> & C2)
{
    return !(C1==C2);
}



// Useful template operators and functions that work on vectors and points

template <class Numeric>
inline GLvector3<Numeric> operator + (const GLvector3<Numeric> & V1, const GLvector3<Numeric> & V2)
{ 
  return GLvector3<Numeric>(V1.x()+V2.x(), V1.y()+V2.y(), V1.z()+V2.z()); 
}

template <class Numeric>
inline GLvector3<Numeric> operator - (const GLvector3<Numeric> & V1, const GLvector3<Numeric> & V2)
{ 
  return GLvector3<Numeric>(V1.x()-V2.x(), V1.y()-V2.y(), V1.z()-V2.z()); 
}

template <class Numeric>
inline GLpoint4<Numeric> operator + (const GLpoint4<Numeric> & P, const GLvector3<Numeric> & V) 
{ 
  return GLpoint4<Numeric>(P.x()+V.x(), P.y()+V.y(), P.z()+V.z()); 
}

template <class Numeric> // Cross product
inline GLvector3<Numeric> operator * (const GLvector3<Numeric> & V1, const GLvector3<Numeric> & V2) 
{ 
  return GLvector3<Numeric>(V1.y()*V2.z()-V2.y()*V1.z(), V2.x()*V1.z()-V1.x()*V2.z(), V1.x()*V2.y()-V2.x()*V1.y()); 
}

template <class Numeric> // Explicit Cross Product
inline GLvector3<Numeric> CrossProduct(const GLvector3<Numeric> & V1,const GLvector3<Numeric> & V2)
{
   return GLvector3<Numeric>(V1.y()*V2.z()-V2.y()*V1.z(), V2.x()*V1.z()-V1.x()*V2.z(), V1.x*V2.y()-V2.x()*V1.y()); 
}

template <class Numeric>
inline Numeric DotProduct(const GLvector3<Numeric> & V1,const GLvector3<Numeric> & V2)
{ 
    return V1.x()*V2.x()+V1.y()*V2.y()+V1.z()*V2.z(); 
}

template <class Numeric>
GLvector3<Numeric> NormalizedCrossProduct(const GLvector3<Numeric> & V1,const GLvector3<Numeric> & V2)
{
      GLvector3<Numeric> V(V1.y()*V2.z()-V2.y()*V1.z(), V2.x()*V1.z()-V1.x()*V2.z(), V1.x()*V2.y()-V2.x()*V1.y());  
      V.Normalize();
      return V;
}

template <class Numeric>
GLvector3<Numeric> operator - (const GLpoint4<Numeric> & C1, const GLpoint4<Numeric> & C2) throw(std::runtime_error)
{ 
      ASSERT(C1.w()!=0 && C2.w()!=0); // Check that points are not at infinity
      if (C1.w()==0 || C2.w()==0) 
          throw std::runtime_error("Cannot subtract points if any of them is at infinity");
      else if (C1.w()==1 && C2.w()==1) 
          return GLvector3<Numeric>(C1.x()-C2.x(), C1.y()-C2.y(), C1.z()-C2.z());
      else
          return GLvector3<Numeric>(C1.x()/C1.w()-C2.x()/C2.w(), C1.y()/C1.w()-C2.y()/C2.w(), C1.z()/C1.w()-C2.z()/C2.w());
}

#endif