// File MyGraph.cpp

#include "stdafx.h"
#include "oglctrl.h" 
#include "oglutil.h" 
#include "MyGraph.h"

CMyGraph::CMyGraph()
         :RotationX(0),RotationY(0)
{
    CreateEx(0,"MyOpenGL","Sample3D",
             WS_OVERLAPPEDWINDOW,
             100,100,300,300,0,0);
}

void CMyGraph::PostNcDestroy()
{
   delete this;
}

BEGIN_MESSAGE_MAP(CMyGraph,COpenGLCtrl)
    ON_WM_LBUTTONDOWN()
    ON_WM_LBUTTONUP()
    ON_WM_MOUSEMOVE()
END_MESSAGE_MAP()


// Setup ugly light and material properties (I am no artist)
void CMyGraph::OnCreateRC()
{
	glClearColor(0.0f,0.0f,0.0f,0.0f);
	glClearDepth(1.0f);
	glEnable(GL_DEPTH_TEST);

	glDepthFunc(GL_LEQUAL);
	glEnable(GL_NORMALIZE);   

    GLfloat glfLightModel[] = { 0.3f, 0.3f, 0.3f, 1.0f };
	GLfloat glfLightAmbient[] = { 0.2f, 0.2f, 0.2f, 1.0f };
	GLfloat glfLightDiffuse[] = { 0.5f, 0.5f, 0.5f, 1.0f };
	GLfloat glfLightSpecular[] = { 0.7f, 0.7f, 0.7f, 1.0f };
	GLfloat glfLightPosition[] = { 4.0f, 10.0f, 20.0f, 0.0f };

	glMaterialf(GL_FRONT_AND_BACK,GL_SHININESS,20.0f);
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, glfLightModel);
	glLightfv (GL_LIGHT0, GL_AMBIENT, glfLightAmbient);
	glLightfv (GL_LIGHT0, GL_DIFFUSE, glfLightDiffuse);
	glLightfv (GL_LIGHT0, GL_SPECULAR, glfLightSpecular);
	glLightfv (GL_LIGHT0, GL_POSITION, glfLightPosition);

	GLfloat glfMaterialAmbient[] = { 0.8f,0.2f,0.2f,1.0f};
	GLfloat glfMaterialDiffuset[] = { 1.0f,0.4f,0.4f,1.0f};
	GLfloat glfMaterialSpecular[] = { 0.5f,0.1f,0.1f,1.0f};

	glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT,glfMaterialAmbient);
	glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,glfMaterialDiffuset);
	glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,glfMaterialSpecular);
	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE,GL_TRUE); 

	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
}

void CMyGraph::OnRender()
{
    // Rotate the model
    glRotatef((GLfloat)RotationY,0.0f,1.0f,0.0f);
    glRotatef((GLfloat)RotationX,1.0f,0.0f,0.0f);

    // Specify my four points with template classes
	GLpoint4<GLfloat> P0(0.0F,1.0F,0.0F);
	GLpoint4<GLfloat> P1(0.0F,0.0F,-1.0F);
	GLpoint4<GLfloat> P2(1.0F,0.0F,0.0F);
	GLpoint4<GLfloat> P3(0.0F,0.0F,1.0F);
	GLpoint4<GLfloat> P4(-1.0F,0.0F,0.0F);
     
    // Draw the pyramid, with its normals
    glBegin(GL_TRIANGLES);
	
	   glNormal3fv((P0-P1)*(P1-P2)); 
	   glVertex3fv(P0);
	   glVertex3fv(P1);
	   glVertex3fv(P2);

	   glNormal3fv((P0-P2)*(P2-P3)); 
	   glVertex3fv(P0);
	   glVertex3fv(P2);
	   glVertex3fv(P3);

       glNormal3fv((P0-P3)*(P3-P4)); 
	   glVertex3fv(P0);
	   glVertex3fv(P3);
	   glVertex3fv(P4);

       glNormal3fv((P0-P4)*(P4-P1)); 
	   glVertex3fv(P0);
	   glVertex3fv(P4);
	   glVertex3fv(P1);

    glEnd();

}

void CMyGraph::OnDestroyRC()
{
}

void CMyGraph::OnViewport(int cx, int cy)
{
    glViewport(0,0,cx,cy);
}

void CMyGraph::OnViewing()
{
    gluLookAt(2,2,2,0,0,0,0,1,0);
}

void CMyGraph::OnProjection(GLdouble AspectRatio)
{
    gluPerspective(45,AspectRatio,1,10);
}

void CMyGraph::OnLButtonDown(UINT nFlags, CPoint point)
{
    SetCapture();
    MouseDown=point;
    SetCursor(LoadCursor(NULL,IDC_SIZEALL));
}

void CMyGraph::OnLButtonUp(UINT nFlags, CPoint point)
{
    ReleaseCapture();
}

void CMyGraph::OnMouseMove(UINT nFlags, CPoint point)
{
    CRect R;
    GetClientRect(&R);

    if (!GetCapture() || R.IsRectEmpty()) 
        return;

    // Implement wild rotation 
    RotationY=(RotationY+((point.x-MouseDown.x)*360/R.Width()))%360; 
    RotationX=(RotationX+((point.y-MouseDown.y)*360/R.Height()))%360; 

    MouseDown=point;
    InvalidateRect(NULL);
}
    